VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "CParser"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

Private Enum EPrintPos
    EPP_Normal
    EPP_Lower
    EPP_Higher
End Enum

Private mLexer      As CScanner

Private mOutput     As PictureBox
Private mMarkUp     As String

'##########################################################################
'# Properties                                                             #
'##########################################################################

'-------------------------------------------------
'Property: OutputDevice
'Picturebox, auf der der Text ausgegeben werden soll
'-------------------------------------------------
Public Property Get OutputDevice() As PictureBox
    Set OutputDevice = mOutput
End Property

Public Property Set OutputDevice(RHS As PictureBox)
    Set mOutput = RHS
End Property

'-------------------------------------------------
'Property: Markup
'Markuplanguage die dargestellt werden soll
'-------------------------------------------------
Public Property Get MarkUp() As String
    MarkUp = mMarkUp
End Property

Public Property Let MarkUp(RHS As String)
    mMarkUp = RHS
End Property

'-------------------------------------------------
'Property: Unmarked
'Gibt den Text ohne Tags aus
'-------------------------------------------------
Public Property Get Unmarked() As String
    Dim CurTok  As TToken

    If mOutput Is Nothing Then _
        Exit Sub

    mLexer.InputStream = mMarkUp

    Do Until mLexer.GetNextToken(False).TT_Type = ETT_EOS
        CurTok = mLexer.GetNextToken

        Select Case CurTok.TT_Type
            Case ETT_Text
                Unmarked = Unmarked & CurTok.TT_Text
            Case ETT_LineBreak
                Unmarked = Unmarked & vbCrLf
        End Select
    Loop
End Property

'##########################################################################
'# Subs/Functions                                                         #
'##########################################################################

Private Sub Class_Initialize()
    Set mLexer = New CScanner
End Sub

Private Sub Class_Terminate()
    Set mLexer = Nothing
End Sub

'-------------------------------------------------
'Parse
'Stellt den Text formatiert dar
'-------------------------------------------------
Public Sub Parse()
    Dim CurTok  As TToken
    Dim Pos     As EPrintPos

    If mOutput Is Nothing Then _
        Exit Sub

    mLexer.InputStream = mMarkUp

    Do Until mLexer.GetNextToken(False).TT_Type = ETT_EOS
        CurTok = mLexer.GetNextToken

        Select Case CurTok.TT_Type
            Case ETT_Text:
                Select Case Pos
                    Case EPP_Normal:    Call PrintNormal(CurTok.TT_Text)
                    Case EPP_Higher:    Call PrintHigher(CurTok.TT_Text)
                    Case EPP_Lower:     Call PrintLower(CurTok.TT_Text)
                End Select
            Case ETT_LineBreak:         mOutput.Print ""
            Case ETT_BoldStart:         mOutput.FontBold = True
            Case ETT_BoldStop:          mOutput.FontBold = False
            Case ETT_UnderlineStart:    mOutput.FontUnderline = True
            Case ETT_UnderlineStop:     mOutput.FontUnderline = False
            Case ETT_ItalicStart:       mOutput.FontItalic = True
            Case ETT_ItalicStop:        mOutput.FontItalic = False
            Case ETT_HigherStart:       Pos = EPP_Higher
            Case ETT_HigherStop:        Pos = EPP_Normal
            Case ETT_LowerStart:        Pos = EPP_Lower
            Case ETT_LowerStop:         Pos = EPP_Normal
            Case ETT_FontStart:         mOutput.Font.Name = CurTok.TT_Font
            Case ETT_FontStop:          mOutput.Font.Name = "MS Sans Serif"
            Case ETT_FontSizeStart:     mOutput.Font.Size = CurTok.TT_FontSize
            Case ETT_FontSizeStop:      mOutput.Font.Size = 8
            Case ETT_ColorStart:        mOutput.ForeColor = CurTok.TT_Color
            Case ETT_ColorStop:         mOutput.ForeColor = 0
        End Select
    Loop
End Sub

'-------------------------------------------------
'PrintNormal
'Schreibt den Text in normaler Zeilenhhe
'-------------------------------------------------
Private Sub PrintNormal(Text As String)
    Dim OldX    As Long
    Dim OldY    As Long

    OldX = mOutput.CurrentX + mOutput.TextWidth(Text)
    OldY = mOutput.CurrentY

    mOutput.Print Text

    mOutput.CurrentY = OldY
    mOutput.CurrentX = OldX
End Sub

'-------------------------------------------------
'PrintHigher
'Schreibt den Text hhergestellt
'-------------------------------------------------
Private Sub PrintHigher(Text As String)
    Dim OldSize     As Double
    Dim OldY        As Long
    Dim OldX        As Long

    OldSize = mOutput.Font.Size

    OldX = mOutput.CurrentX + mOutput.TextWidth(Text)
    OldY = mOutput.CurrentY

    mOutput.Font.Size = OldSize * 0.6

    mOutput.Print Text

    mOutput.CurrentY = OldY
    mOutput.CurrentX = OldX
    mOutput.Font.Size = OldSize
End Sub

'-------------------------------------------------
'PrintLower
'Schreibt den Text tiefergestellt
'-------------------------------------------------
Private Sub PrintLower(Text As String)
    Dim OldSize     As Double
    Dim OldY        As Long
    Dim OldX        As Long

    OldSize = mOutput.Font.Size

    OldX = mOutput.CurrentX + mOutput.TextWidth(Text)
    OldY = mOutput.CurrentY

    mOutput.CurrentY = OldY + mOutput.TextHeight("A")
    mOutput.Font.Size = OldSize * 0.6
    mOutput.CurrentY = mOutput.CurrentY - mOutput.TextHeight("A")

    mOutput.Print Text

    mOutput.CurrentY = OldY
    mOutput.CurrentX = OldX
    mOutput.Font.Size = OldSize
End Sub
